--
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library machxo2;
use machxo2.all;

--
ENTITY top IS
   PORT (
		M_TMS         		: IN std_logic;
		M_TCK         		: IN std_logic;
		M_TDI         		: IN std_logic;
		M_TDO         		: OUT std_logic;
	
		C_TMS         		: OUT std_logic;
		C_TCK         		: OUT std_logic;
		C_TDI         		: OUT std_logic;
		C_TDO         		: IN std_logic;

		FMC_TMS        		: OUT std_logic;
		FMC_TCK        		: OUT std_logic;
		FMC_TDI        		: OUT std_logic;
		FMC_TDO        		: IN std_logic;

		-- Pushbuttons
		S1                  : IN std_logic;
		S2                  : IN std_logic;
		
		-- TE70xx control pins, 3.3V IN power domain
		NOSEQ               : INOUT std_logic;
		EN1                 : OUT std_logic;
		RESIN               : OUT std_logic;
		
		MODE                : OUT std_logic; -- SCL
		PGOOD               : INOUT std_logic; -- SDA
		
		-- HDMI	
		SDA                 : INOUT std_logic;
		SCL                 : INOUT std_logic;
		
		
		-- 3.3V out, or PGOOD domain, should be driven low not powered?
		MIO10               : INOUT std_logic; --	 	
		MIO11               : INOUT std_logic; --
		MIO12               : INOUT std_logic; --
		MIO13               : INOUT std_logic; --
		MIO14               : INOUT std_logic; -- UART0.RX << BDBUS0
		MIO15               : INOUT std_logic; -- UART0.TX >> BDBUS1
		
		ADBUS4              : OUT std_logic; 
		ADBUS7              : IN std_logic; 

		
		ACBUS4              : IN std_logic; 
		ACBUS5              : IN std_logic; 
		
		BDBUS0              : INOUT std_logic; -- TCK in or TX from FTDI
		BDBUS1              : INOUT std_logic; -- TDI in or RX to FTDI
		BDBUS2              : INOUT std_logic; -- TDO out
		BDBUS3              : INOUT std_logic; -- TMS in
		
		--
	    ext_reset_n		    : IN std_logic; 

		-- misc control and status
		USB_OC              : IN std_logic;
		SD_DETECT           : IN std_logic;
		SD_WP               : IN std_logic;
		SEL_SD              : OUT std_logic;
		

		VID0                : OUT std_logic;
		VID1                : OUT std_logic;
		VID2                : OUT std_logic;
		EN_FMC              : OUT std_logic;
		PG_C2M              : OUT std_logic;
		FMC_PRSNT           : IN std_logic;
		POK_FMC             : IN std_logic;
		
		-- LED's in MAGJACK
		PHY_LED1            : OUT std_logic;
		PHY_LED2            : OUT std_logic;

		-- extra pins in MIO bank1 PMod J2
		PX6                 : INOUT std_logic;
		PX7                 : INOUT std_logic;

	    -- 3.3V IN powered
		LED1                : OUT std_logic;
		LED2                : OUT std_logic;
		LED3                : OUT std_logic;
		LED4                : OUT std_logic;
		
		-- FMC VADJ powered
		LED5                : OUT std_logic;
		LED6                : OUT std_logic;
		LED7                : OUT std_logic;
		LED8                : OUT std_logic;

		dummy               : OUT std_logic
      );
END ENTITY top;
--
ARCHITECTURE translated OF top IS


COMPONENT OSCH
	-- synthesis translate_off
	GENERIC (NOM_FREQ: string := "24.18");
	-- synthesis translate_on
	PORT ( STDBY :IN std_logic;
	OSC :OUT std_logic;
	SEDSTDBY :OUT std_logic);
END COMPONENT OSCH;

 attribute NOM_FREQ : string;
 attribute NOM_FREQ of MyInternalOSC : label is "24.18";

component I2C_to_GPIO8
    port (
		sda: inout  std_logic; 
        sclk: in  std_logic;
		
        GPIO_input: in  std_logic_vector(7 downto 0);
        GPIO_output: out  std_logic_vector(7 downto 0)
		);
end component;


	COMPONENT PORESET
	PORT(
		SYSRST : IN std_logic;
		CLK : IN std_logic;          
		RST : OUT std_logic
		);
	END COMPONENT;


signal USB_JTAG_RST: std_logic;

signal clk: std_logic;
signal cnt: std_logic_vector(31 downto 0);
signal FMC_JTAGEN: std_logic;

signal POR_DONE: std_logic;
signal por_cnt: std_logic_vector(31 downto 0);

signal POR_DELAYED: std_logic;


signal GPIO_input: std_logic_vector(31 downto 0);
signal GPIO_output: std_logic_vector(31 downto 0);

signal osc_clk: std_logic;

-- power ON, drive all output low if no powered!
signal PON: std_logic;

signal BOOTMODE: std_logic;

BEGIN
	dummy 		<= SD_WP or USB_OC or S2 or POK_FMC or ADBUS7;
	
	-- When SD card in Slot force BOOTMODE to SD!
	BOOTMODE <= '0' when SD_DETECT='0' else '1';
	
	-- Select MIO mux to SD or Pmod
	-- Select SD card (not pmod J2)
	-- note: SD card in pmod J2 can be used as non bootable SD or eMMC card (not bootable)
	SEL_SD   <= BOOTMODE;	

	--MODE 		<= scl_out when i2c_mode='1' else BOOTMODE;
	MODE <= SD_DETECT;

	-- Pmod MIO bank0 multiplexer
	--
	-- Connect ZYNQ UART to FTDI Channel B UART
	--
	MIO14 <= 'Z' when PON = '0' else BDBUS0;
	BDBUS1 <= MIO15;

    --
	USB_JTAG_RST <= ACBUS4 or ACBUS5;
	
	-- Reset if button S1 or USB forced RST
	RESIN 	<= S1 and USB_JTAG_RST;

	PON 	<= S2 and POR_DELAYED;
	EN1 	<= PON;
	
	FMC_JTAGEN  <= '1' when FMC_PRSNT='0' else '0';
	
	clk <= osc_clk;
	
	LED1 		<= NOSEQ; -- 
	LED2 		<= S1; --M_TMS; -- 
	LED3 		<= S2; --M_TCK; -- 
	LED4 		<= M_TDI; --


	LED5 <= not SCL;
	

    NOSEQ <= 'Z';
	PX6 <= 'Z';
	PX7 <= 'Z';

	-- Ethernet MAGJACK	LEDs
	PHY_LED1 	<= NOSEQ; -- 
	PHY_LED2 	<= GPIO_Output(0);   -- 

	-- JTAG Chain multiplexer
    C_TMS 		<= M_TMS and PON;
    C_TCK 		<= M_TCK and PON;
    C_TDI 		<= M_TDI and PON;

    M_TDO 		<= C_TDO when FMC_JTAGEN='1' else FMC_TDO;
	ADBUS4 		<= M_TCK;

    FMC_TMS 		<= M_TMS when FMC_JTAGEN='1' else '0';
    FMC_TCK 		<= M_TCK when FMC_JTAGEN='1' else '0';
    FMC_TDI 		<= M_TDI when FMC_JTAGEN='1' else '0';

	PGOOD       <= 'Z';

	-- Power/Reset Control

	-- FMC Related signals
	PG_C2M	 	<= GPIO_output(3) and PON;
	EN_FMC 		<= GPIO_output(7) and PON;-- xor FMC_PRSNT; -- default ON when FMC not present, and OFF when FMC present
	
	-- 000 = 3.3V
	VID0  		<= GPIO_output(4);
	VID1  		<= GPIO_output(5);
	VID2  		<= GPIO_output(6);

--
-- XO2 Internal oscillator
--
MyInternalOSC: OSCH
		-- synthesis translate_off
		GENERIC MAP ( NOM_FREQ => "24.18" )
		-- synthesis translate_on
		PORT MAP ( 
			STDBY		=> '0',
			OSC			=> osc_clk,
			SEDSTDBY	=> open
		);
	

	GPIO_input(0) <= FMC_PRSNT;
	GPIO_input(1) <= POK_FMC;
    GPIO_input(2) <= SD_DETECT;
    GPIO_input(3) <= SD_WP;
    GPIO_input(4) <= USB_OC;
	
	
	

My_GPIO: I2C_to_GPIO8 port map (
		sda 	=> SDA,
        sclk 	=> SCL,
		
        GPIO_input =>  GPIO_input(7 downto 0),
        GPIO_output => GPIO_output(7 downto 0)
		);


Inst_PORESET: PORESET PORT MAP( 		
	SYSRST => ext_reset_n, 		
	RST => POR_DONE, 		
	CLK => Clk 	);


	process (POR_DONE, Clk)
	begin
		if POR_DONE = '0' then
			POR_DELAYED <= '0'; -- 
			por_cnt <= X"00000000";
		elsif Clk'event and Clk = '1' then
			por_cnt <= por_cnt + X"00000001";
			if por_cnt(23)='1' then
				POR_DELAYED <= '1';
			end if;
		end if;
	end process;





END ARCHITECTURE translated;